Tekagis = {
    UPGRADE_CHANCE = 10, -- 1 in # chance item is upgraded
    SPECIAL_CHANCE = 100000, -- 1 in # chance item is special
    DurabilityFixTable = {
        "Base.Axe",
        "Base.BaseballBat",
        "Base.BaseballBatNails",
        --"Base.ButterKnife",
        "Base.Hammer",
        "Base.KitchenKnife",
        "Base.Poolcue",
        "Base.Screwdriver",
        "Base.Sledgehammer",
        "Base.AxeStone",
        "Base.HuntingKnife",
        "Base.IcePick",
        "Base.LetterOpener",
        "Base.Crowbar",
    },
    UpgradedItemsTable = { },
    SpecialItemsTable = { },
}


Tekagis.upgradeDurabilities = function()
    for _, item in ipairs(Tekagis.DurabilityFixTable) do
        local scriptItem = getScriptManager():FindItem(item)
        if scriptItem then
            scriptItem:setConditionLowerChance(scriptItem:getConditionLowerChance() * 2)
        end
    end
end

Tekagis.findAllUpgradesForItem = function(item)
    return Tekagis.UpgradedItemsTable[item:getFullType()]
end
Tekagis.findUpgradeForItem = function(item, name)
    local tbl = Tekagis.findAllUpgradesForItem(item)
    if not tbl then return end
    for _, upgrade in ipairs(tbl) do
        if upgrade.absolute and upgrade.absolute.Name == name then
            return upgrade
        end
    end
end

Tekagis.findAllSpecialsForItem = function(item)
    return Tekagis.SpecialItemsTable[item:getFullType()]
end
Tekagis.findSpecialForItem = function(item, name)
    local tbl = Tekagis.findAllSpecialsForItem(item)
    if not tbl then return end
    for _, upgrade in ipairs(tbl) do
        if upgrade.absolute and upgrade.absolute.Name == name then
            return upgrade
        end
    end
end


Tekagis.upgradeItem = function(item, upgrade, ignoreUpgradeFlag)
    if not item or not upgrade then return end
    if item:getModData().UPGRADE and not ignoreUpgradeFlag then return end
    item:getModData().UPGRADE = true
    
    if upgrade.absolute then
        for key, value in pairs(upgrade.absolute) do
            --print("key: "..tostring(key) .." value: "..tostring(value))
            item["set" .. key](item, value)
        end
    end
    if upgrade.multiplier then
        for key, value in pairs(upgrade.multiplier) do
            item["set" ..key](item, item["get" ..key](item) * value)
        end
    end
    if upgrade.relative then
        for key, value in pairs(upgrade.relative) do
            item["set" ..key](item, item["get" ..key](item) + value)
        end
    end
end

Tekagis.checkForUpgradeInContainer = function(container)
    -- see if any of our items that spawned have possible upgrades
    for itemName, upgrades in pairs(Tekagis.UpgradedItemsTable) do repeat
        local items = container:FindAll(itemName)
        if not items then break end
        for i=1,items:size() do repeat
            if ZombRand(Tekagis.UPGRADE_CHANCE) +1 > 1 then break end
            Tekagis.upgradeItem(items:get(i-1), upgrades[ZombRand(#upgrades) + 1], false)

        until true end
    until true end
    
    -- see if any of our items that spawned have possible special versions
    for itemName, upgrades in pairs(Tekagis.SpecialItemsTable) do repeat
        local items = container:FindAll(itemName)
        if not items then break end

        for i=1,items:size() do repeat
            
            if ZombRand(Tekagis.SPECIAL_CHANCE) +1 > 1 then break end
            local item = items:get(i-1)
            if not item then break end
            Tekagis.upgradeItem(item, upgrades[ZombRand(#upgrades) + 1], true)
            -- set condition to max since this is a special
            item:setCondition(item:getConditionMax())
        until true end
    until true end
end

Tekagis.findNearbyGenerator = function(square, xydistance, zdistance)
    local zoneSize = 30; -- size of zone to check; increase if you have troubles with gas pumps distance
    if not square then return end
    local xMin = square:getX() - xydistance
    local xMax = square:getX() + xydistance
    local yMin = square:getY() - xydistance
    local yMax = square:getY() + xydistance
    local zMin = square:getZ() - zdistance
    local zMax = square:getZ() + zdistance
    --print("seaching for generator...")
    --print("starting search at: "..tostring(square:getX())..", "..tostring(square:getY())..", "..tostring(square:getZ()))
    
    if zMin < 0 then zMin = 0 end
    for z = zMin, zMax do
        for x = xMin, xMax do
            for y = yMin, yMax do
                --print("checking "..tostring(x)..", "..tostring(y)..", "..tostring(z))
                local this = square:getCell():getGridSquare(x, y, z)
                --if not this then print("no square?!?!?!") end
                --if this and this:getGenerator() then print("found a gen....") end
                if this and this:getGenerator() and this:getGenerator():isActivated() then
                    --print("Found working gen!")
                    return this
                end
            end
        end
    end
end

bit32 = {
    bxor = function(a,b)--Bitwise xor
        local p,c=1,0
        while a>0 and b>0 do
            local ra,rb=a%2,b%2
            if ra~=rb then c=c+p end
            a,b,p=(a-ra)/2,(b-rb)/2,p*2
        end
        if a<b then a=b end
        while a>0 do
            local ra=a%2
            if ra>0 then c=c+p end
            a,p=(a-ra)/2,p*2
        end
        return c
    end,

    bor = function(a,b)--Bitwise or
        local p,c=1,0
        while a+b>0 do
            local ra,rb=a%2,b%2
            if ra+rb>0 then c=c+p end
            a,b,p=(a-ra)/2,(b-rb)/2,p*2
        end
        return c
    end,

    bnot = function(n)
        local p,c=1,0
        while n>0 do
            local r=n%2
            if r<1 then c=c+p end
            n,p=(n-r)/2,p*2
        end
        return c
    end,

    band = function(a,b)--Bitwise and
        local p,c=1,0
        while a>0 and b>0 do
            local ra,rb=a%2,b%2
            if ra+rb>1 then c=c+p end
            a,b,p=(a-ra)/2,(b-rb)/2,p*2
        end
        return c
    end,

    
    lshift = function(x, by)
      return x * 2 ^ by
    end,

    rshift = function(x, by)
      return math.floor(x / 2 ^ by)
    end,
}


Tekagis.upgradeDurabilities()

--print("------------------------")
--print(BitXOR(7,2))
--print(BitOR(7,2))
--print(BitNOT(7,2)) -- assert(bit32.bnot(x) == (-1 - x) % 2^32)
--print(BitNOT(5,2))

